#include "textinputnode.h"

#include <iostream>

#include "SparrowInput/input.h"

#include "engine.h"
#include "tools/font.h"
#include "resourcemanager.h"

TextInputNode::TextInputNode(glm::vec2 dimension):
    m_dimension(dimension),
    m_font_size(16.f),
    m_text_color(glm::vec3(1,1,1)),
    m_cursor_pos(0)
{
  //  Font *shellfont = RESOURCE_GET(Font,"shellfont");
    Mesh* mesh = new Mesh();
    mesh->addRectangle2D(glm::vec2(0),glm::vec2(2, m_font_size));
    PhongMaterial* mat = new PhongMaterial();
    mat->diffuse = glm::vec3(1,1,1);
    mesh->setMaterial(mat);
    mesh->setDepth(30);
    mesh->initGL();
    m_cursor_mesh = new MeshNode(mesh,false);
    addChild(m_cursor_mesh);
}

void TextInputNode::update()
{
    if(!m_hasFocus) return;

    std::wstring text = getEngine().getInput()->getText();
    bool input_string_updated = false;
    bool cursor_pos_updated = false;

    auto input = getEngine().getInput();
    for(auto action : input->getActions()){
        if (action.action == m_move_cursor_left){
            if (m_cursor_pos > 0){
                m_cursor_pos--;
                cursor_pos_updated=true;
            }
        }
        else if(action.action == m_move_cursor_right){
            if(m_cursor_pos < m_text.length()){
                m_cursor_pos++;
                cursor_pos_updated=true;
            }
        }
    }

    for(unsigned int i = 0 ; i < text.length() ; i++){
        char c = text[i];
        switch(c){
        case 8:
            if(m_cursor_pos > 0)
                m_text.erase(--m_cursor_pos,1);
            input_string_updated = true;
            cursor_pos_updated=true;
            break;
        case 13:
            if (!m_text.empty())
            {
                if(m_callback)
                    m_callback->exec();
                input_string_updated = true;
                m_cursor_pos = 0;
                cursor_pos_updated=true;
            }
            break;
        case 127:
            m_text.erase(m_cursor_pos,1);
            input_string_updated = true;
            break;
        default:
            m_text.insert(m_cursor_pos++,std::string(1,c));
            input_string_updated = true;
            cursor_pos_updated=true;
        }
    }
    Font *shellfont = RESOURCE_GET(Font,"shellfont");
    if(cursor_pos_updated){
        m_cursor_mesh->moveTo2D(glm::vec2(m_cursor_pos*shellfont->getXAdvance()*(m_font_size/shellfont->getLineHeight()),0));
    }

    if(input_string_updated)
    {
        if(m_text_mesh){
            this->removeChild(m_text_mesh);
            delete(m_text_mesh);
        }
        m_text_mesh = shellfont->getTextNode(m_text,m_text_color,m_font_size,false);
        if(m_text_mesh){
            m_text_mesh->setTransform(glm::mat4());
            m_text_mesh->setDepth(15);
            this->addChild(m_text_mesh);
            m_text_mesh->setVisible(true);
        }
    }
    GUINode::update();
}

void TextInputNode::setFocus(bool focus)
{
    m_hasFocus = focus;
    m_cursor_mesh->setVisible(focus);
}

void TextInputNode::setTextColor(glm::vec3 color)
{
    m_text_color = color;
}

std::string TextInputNode::getText()
{
    return m_text;
}




